// export
class Viewport {
    constructor() {
        this.visible = [0, 0, 0, 0]//fromX, fromY, toX, toY
    }
    setVisible(fromX, fromY, toX, toY) {
        this.visible = [fromX, fromY, toX, toY]
    }
    transform(data, [left, top, width, height]) {
        let result = []
        let length = data.length / 2
        let visibleLeft = this.visible[0]
        let visibleBottom = this.visible[1]
        let visibleWidth = this.visible[2] - visibleLeft
        let visibleHeight = this.visible[3] - visibleBottom

        let screenLeft = left
        let screenTop = top
        let screenWidth = width
        let screenHeight = height

        for (let i = 0; i < length; i++) {
            let x = screenLeft + (data[2 * i] - visibleLeft) / visibleWidth * screenWidth
            let y = screenTop + screenHeight - (data[2 * i + 1] - visibleBottom) / visibleHeight * screenHeight
            result.push(x)
            result.push(y)
        }
        return result
    }
    transformX(x, [left, top, width, height]) {

        let visibleLeft = this.visible[0]
        let visibleWidth = this.visible[2] - visibleLeft

        let screenLeft = left
        let screenWidth = width

        return screenLeft + (x - visibleLeft) / visibleWidth * screenWidth
    }
    transformY(y, [left, top, width, height]) {

        let visibleBottom = this.visible[1]
        let visibleHeight = this.visible[3] - visibleBottom

        let screenTop = top
        let screenHeight = height
        return screenTop + screenHeight - (y - visibleBottom) / visibleHeight * screenHeight
    }
    offset(x, y) {
        let [fromX, fromY, toX, toY] = this.visible
        this.visible = [fromX + x, fromY + y, toX + x, toY + y]
    }
    center() {
        let [fromX, fromY, toX, toY] = this.visible
        return [fromX + (toX - fromX) * 0.5, fromY + (toY - fromY) * 0.5]
    }
    zoom(delta) {
        let [fromX, fromY, toX, toY] = this.visible
        let [x, y] = this.center();
        let ratio = 1 + delta
        let left = x - (x - fromX) * ratio;
        let right = left + (toX - fromX) * ratio;
        let bottom = y - (y - fromY) * ratio;
        let top = bottom + (toY - fromY) * ratio;
        this.visible = [left, bottom, right, top]
    }
}