// import { CanvasDrawingElement } from './canvasDrawingElement.js'
// import { HorizontalAxisDrawing } from './horizontalAxisDrawing.js'
// import { VerticalAxisDrawing } from './verticalAxisDrawing.js'
// import { ChartArea } from '../chart/chartArea.js'
// export 
class CanvasDrawing {
    constructor(width, height) {
        var canvas = this.canvas = document.createElement("canvas")
        var view = this.view = document.createElement("canvas")
        canvas.width = width
        canvas.height = height
        view.width = width
        view.height = height

        this.width = width
        this.height = height

        this.context = canvas.getContext("2d")
        this.viewContext = view.getContext("2d")

        this.screens = {
            [ChartArea.plot]: [50, 0, width - 50, height - 50],
            [ChartArea.xAxis]: [50, height - 50, width - 50, 50],
            [ChartArea.yAxis]: [0, 0, 50, height - 50],
        }
    }
    init(dom, chart) {
        this.chart = chart
        dom.appendChild(this.view)

        document.addEventListener("mousedown", e => {
            if (e.target == this.view) e.preventDefault() 
            var mouseEventArgs = this.createMouseEventArgs(e)
            chart.mouseDown(mouseEventArgs)
        })
        document.addEventListener("mouseup", e => {
            if (e.target == this.view) e.preventDefault()
            var mouseEventArgs = this.createMouseEventArgs(e)
            chart.mouseUp(mouseEventArgs)
        })
        document.addEventListener("mousemove", e => {
            if (e.target == this.view) e.preventDefault()
            var mouseEventArgs = this.createMouseEventArgs(e)
            chart.mouseMove(mouseEventArgs)
        })
        document.addEventListener("mousewheel", e => {
            if (e.target == this.view) e.preventDefault()
            var mouseEventArgs = this.createMouseEventArgs(e)
            chart.mouseWheel(mouseEventArgs)
        })
    }
    createMouseEventArgs(e) {
        let offsetX = this.view.offsetLeft
        let offsetY = this.view.offsetTop
        let x = e.pageX - offsetX
        let y = e.pageY - offsetY
        let area = this.findArea(x, y)
        let screen = this.getScreen(area)
        return new MouseEventArgs(x, y, area, screen, e)
    }
    renderChart() {
        let chart = this.chart
        let context = this.context
        context.save()
        context.fillStyle = "#ffffff"
        context.fillRect(0, 0, this.width, this.height)
        context.restore()
        for (let area of this.getArea()) {
            let screen = this.getScreen(area)
            for (let element of this.getElements(chart, area)) {
                context.save()
                element.render(context, screen)
                context.restore()
            }
            this.viewContext.drawImage(this.canvas, ...screen, ...screen)
        }
    }
    findArea(x, y) {
        for (let area of this.getArea()) {
            let [x2, y2, width, height] = this.getScreen(area)
            if (x >= x2 && x < x2 + width && y >= y2 && y < y2 + height) return area
        }
        return null
    }
    getScreen(area) {
        return this.screens[area]
    }
    * getArea() {
        yield ChartArea.xAxis
        yield ChartArea.yAxis
        yield ChartArea.plot
    }
    * getElements(chart, area) {
        var elements = chart.elements || []
        for (let element of elements.filter(e => e.area == area && this.isDrawElement(e))) {
            yield element
        }
    }
    isDrawElement(element) {
        return [CanvasDrawingElement, HorizontalAxisDrawing, VerticalAxisDrawing]
            .some(type => element instanceof type)
    }
}